import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Random;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import com.taco.data.INoReturnObservableMap;
import com.taco.data.MapUtilities;
import com.taco.swinger.text2gui.DispatchingComponentConverter;
import com.taco.swinger.text2gui.appbase.AppBase;

/** A demo slideshow application that displays pictures of Lily. */
public class SlideShow extends AppBase {
    /** Respond to the user changing the UI. */
    public void propertyChange(PropertyChangeEvent event) { 
        String propertyName = event.getPropertyName();
        Object value = event.getNewValue(); 
        
        if ("pictureIndex".equals(propertyName)) {
            int index = ((Number) value).intValue();   
            
            if ((index >= 0) && (index < _images.length)) {
                _pictureIndex = index;
                _argMap.putNoReturn("icon", _images[index]);   
            }            
        } else if ("back".equals(propertyName)) {
            _changePictureIndex(-1);
        } else if ("forward".equals(propertyName)) {
            _changePictureIndex(1);
        }    
    }    
        
    public static void main(String[] args) {
        start(new SlideShow(), args);
    }
    
    /** Create the argument map used to set the initial parameters of the GUI
     *  and to communicate with it.
     */
    protected INoReturnObservableMap _makeArgMap() {
        super._makeArgMap();
        _argMap.putNoReturn("pictureIndex", new Integer(0));
        _argMap.putNoReturn("icon", _images[0]);
        _argMap.putNoReturn("random", Boolean.TRUE);
        return _argMap;
    }
    
    /** Add listeners for UI changes. */
    protected void _addArgMapListeners() { 
        _argMap.addPropertyChangeListener("pictureIndex", this);
        _argMap.addPropertyChangeListener("back", this);
        _argMap.addPropertyChangeListener("forward", this);
    }
    
    /** Update the picture index by the increment, or if we want to get a
     *  random index, generate a new picture index. Update the argument map
     *  appropriately.
     */
    protected void _changePictureIndex(int increment) {
        if (MapUtilities.getBooleanValue(_argMap, "random")) {
            // Generate a NEW index
            int pictureIndex = -1;
            do {
                pictureIndex = _random.nextInt(_images.length);
            } while (pictureIndex == _pictureIndex);
            
            _pictureIndex = pictureIndex;                                
        } else {                        
            _pictureIndex = (_pictureIndex + increment) % _images.length;
            if (_pictureIndex < 0) {
                _pictureIndex += _images.length;
            }            
        }
        _argMap.putNoReturn("pictureIndex", new Integer(_pictureIndex));        
        _argMap.putNoReturn("icon", _images[_pictureIndex]);        
    }        

    protected int _pictureIndex = 0;
    protected Random _random = new Random();
    
    protected Icon[] _images = new Icon[] {
      new ImageIcon("lily2.png"),
      new ImageIcon("lily1.png"),      
      new ImageIcon("lily3.png"),
      new ImageIcon("lily4.png"),            
    };
}
